@@ -384,7 +384,7 @@
                                   first-token
                                   nil
                                   first-token-end
-                                  nil)
+                                  new-state)
                                  %s-lex-analyzer--buffered-response)
                                 (setq
                                  token-start
